mcl_weather.rain = {
	-- flag to turn on/off extinguish fire for rain
	extinguish_fire = true,

	-- flag useful when mixing weathers
	raining = false
}
mcl_weather.raining = false

mcl_weather.register_weather("rain",{
	light_factor = 0.6,
	cloudcolor= "#5D5D5FE8" ,
	sound = "weather_rain",
	-- 10min - 20min
	min_duration = 600,
	max_duration = 1200,
	transitions = {
		[70] = "none",
		[100] = "thunder",
	},
	skylayer= {{r=0, g=0, b=0},
			{r=85, g=86, b=98},
			{r=135, g=135, b=151},
			{r=85, g=86, b=98},
			{r=0, g=0, b=0}},
	particlespawners = {
		{
			amount = 800,
			time = 0,
			minpos = vector.new(-10,10,-10),
			maxpos =vector.new(10,15,10),
			minvel = vector.new(0,-8,0),
			maxvel = vector.new(0,-12,0),
			minacc = vector.new(0,-20,0),
			maxacc = vector.new(0,-30,0),
			minexptime = 1,
			maxexptime = 2,
			minsize = 0.5,
			maxsize = 5,
			collisiondetection = true,
			collision_removal = true,
			object_collision = true,
			vertical = true,
			glow = 1,
			texture="weather_pack_rain_raindrop_1.png"
		},
		{
			amount = 200,
			time = 0,
			minpos = vector.new(-15,5,-15),
			maxpos =vector.new(15,15,15),
			minvel = vector.new(0,-8,0),
			maxvel = vector.new(0,-12,0),
			minacc = vector.new(0,-20,0),
			maxacc = vector.new(0,-40,0),
			minexptime = 1,
			maxexptime = 1,
			minsize = 0.5,
			maxsize = 3,
			collisiondetection = true,
			collision_removal = true,
			object_collision = true,
			vertical = true,
			glow = 1,
			texture="weather_pack_rain_raindrop_2.png"
		},
		{
			amount = 200,
			time = 0,
			minpos = vector.new(-5,0,-5),
			maxpos = vector.new(5,0.8,5),
			minvel = vector.new(-4, 5,-4),
			maxvel = vector.new(4,10,4),
			minacc = vector.new(0,10,0),
			maxacc = vector.new(0,25,0),
			minexptime = 0.01,
			maxexptime = 0.04,
			minsize = 0.5,
			maxsize = 3,
			collisiondetection = true,
			collision_removal = true,
			object_collision = true,
			vertical = true,
			texture = "weather_pack_rain_raindrop_2.png",
			glow = 0
		}
	},
	start = function() mcl_weather.rain.raining = true end,
	clear = function() mcl_weather.rain.raining = false end,
	at_pos = function(pos)
		local biome=minetest.get_biome_data(pos)
		if mcl_worlds.has_weather(pos) and biome.heat > 95 then
			return true
		end
	end,
	change_at_pos=function(pos)
		local biome=minetest.get_biome_data(pos)
		if mcl_worlds.has_weather(pos) and ( biome.heat < 15 or pos.y > 64 ) then
			return "snow"
		end
	end
})

if mcl_weather.allow_abm then
	-- ABM for extinguish fire
	minetest.register_abm({
		label = "Rain extinguishes fire",
		nodenames = {"mcl_fire:fire"},
		interval = 2.0,
		chance = 2,
		action = function(pos, node, active_object_count, active_object_count_wider)
			-- Fire is extinguished if in rain or one of 4 neighbors is in rain
			if mcl_weather.rain.raining and mcl_weather.rain.extinguish_fire then
				local around = {
					{ x = 0, y = 0, z = 0 },
					{ x = -1, y = 0, z = 0 },
					{ x = 1, y = 0, z = 0 },
					{ x = 0, y = 0, z = -1 },
					{ x = 0, y = 0, z = 1 },
				}
				for a=1, #around do
					local apos = vector.add(pos, around[a])
					if mcl_weather.is_outdoor(apos) then
						minetest.remove_node(pos)
						minetest.sound_play("fire_extinguish_flame", {pos = pos, max_hear_distance = 8, gain = 0.1}, true)
						return
					end
				end
			end
		end,
	})

	-- Slowly fill up cauldrons
	minetest.register_abm({
		label = "Rain fills cauldrons with water",
		nodenames = {"mcl_cauldrons:cauldron", "mcl_cauldrons:cauldron_1", "mcl_cauldrons:cauldron_2"},
		interval = 56.0,
		chance = 1,
		action = function(pos, node, active_object_count, active_object_count_wider)
			-- Rain is equivalent to a water bottle
			if mcl_weather.rain.raining and mcl_weather.is_outdoor(pos) then
				if node.name == "mcl_cauldrons:cauldron" then
					minetest.set_node(pos, {name="mcl_cauldrons:cauldron_1"})
				elseif node.name == "mcl_cauldrons:cauldron_1" then
					minetest.set_node(pos, {name="mcl_cauldrons:cauldron_2"})
				elseif node.name == "mcl_cauldrons:cauldron_2" then
					minetest.set_node(pos, {name="mcl_cauldrons:cauldron_3"})
				elseif node.name == "mcl_cauldrons:cauldron_1r" then
					minetest.set_node(pos, {name="mcl_cauldrons:cauldron_2r"})
				elseif node.name == "mcl_cauldrons:cauldron_2r" then
					minetest.set_node(pos, {name="mcl_cauldrons:cauldron_3r"})
				end
			end
		end
  	})

	-- Wetten the soil
	minetest.register_abm({
		label = "Rain hydrates farmland",
		nodenames = {"mcl_farming:soil"},
		interval = 22.0,
		chance = 3,
		action = function(pos, node, active_object_count, active_object_count_wider)
			if mcl_weather.rain.raining and mcl_weather.is_outdoor(pos) then
				if node.name == "mcl_farming:soil" then
					minetest.set_node(pos, {name="mcl_farming:soil_wet"})
				end
			end
		end
  	})
end